#include <stdlib.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>

#define YUEE_VERSION_MAJOR 0
#define YUEE_VERSION_MINOR 4

#define MAX_LN_LEN 1024

#define MAX_SLEEP_TIME 2000000
#define MIN_SLEEP_TIME 200000
#define DEF_SLEEP_TIME 1000000

int load_MOUSE(unsigned long **p);
int id_cmp( const void *a, const void *b);
int hex2int(const char *s, int *v);
int print_version();
int print_usage();

int main(int argc, char *argv[])
{
	int uusl, usl = DEF_SLEEP_TIME;
	int pfd[2];
	char sz[MAX_LN_LEN], hex[5], *sstr;
	int n, pos, nlist, id = 0;
	unsigned long uid = 0, *uids = NULL;
	
	if( argc == 2 )
	{
		if( strcmp(argv[1], "-version") == 0 )
		{
			print_version();
		}
		
		if( strcmp(argv[1], "--help") == 0 )
		{
			print_usage();
		}
		
		uusl = atoi(argv[1]);
		sprintf(sz, "%d", uusl);
		if( strcmp(sz, argv[1]) == 0 )
		{
			uusl *= 1000;
			
			if( uusl > MAX_SLEEP_TIME )
			{
				usl = MAX_SLEEP_TIME;
			}
			else if( uusl < MIN_SLEEP_TIME )
			{
				usl = MIN_SLEEP_TIME;
			}
			else
			{
				usl = uusl;
			}
		}
	}
	
	nlist  = load_MOUSE(&uids);
	
	if( pipe(pfd) )
	{
		fprintf(stderr, "pipe() failed\n");
		exit(1);
	}
	
	if( dup2(pfd[1], 1) == -1 )
	{
		fprintf(stderr, "dup2() failed\n");
		close(pfd[0]);
		close(pfd[1]);
		exit(1);
	}
	
	while(1)
	{
		system("lsusb");
		n = read(pfd[0], sz, MAX_LN_LEN);
		sz[n] = '\0';
		
		pos = 0;
		while( pos < n )
		{
			sstr = strstr(&sz[pos], "ID");
			if( sstr == NULL )
			{
				break;
			}
			pos = sstr-sz;
			
			pos += 3;
			memcpy(hex, &sz[pos], 4);
			hex[4] = '\0';
			hex2int(hex, &id);
			uid = id << 16;
			
			pos += 5;
			memcpy(hex, &sz[pos], 4);
			hex[4] = '\0';
			hex2int(hex, &id);
			uid += id;
			
			if( uid )
			{
				if( bsearch(&uid, uids, nlist, sizeof(uid), id_cmp) != NULL )
				{
					system("synclient touchpadoff=1");	
				}
				else
				{
					system("synclient touchpadoff=0");
				}
			}
		}
		
		usleep(usl);
	}
	
	exit(0);
}

int load_MOUSE(unsigned long **p)
{
	int n, pos;
	char sz[MAX_LN_LEN], hex[5];
	int id = 0;
	unsigned long uid, *uids;
	FILE *f;
	
	f = fopen("/usr/share/xyuee/MOUSE", "rd");
	if( f == NULL )
	{
		fprintf(stderr, "fopen() failed\n");
		return -1;
	}
	
	n = 0;
	while(1)
	{
		if( fgets(sz, MAX_LN_LEN, f) == NULL )
		{
			break;
		}
		
		if( sz[0] != '#' && strlen(sz) == 10 )
		{
			n += 1;
		}
	}
	
	uids = malloc(sizeof(unsigned long)*n);
	if( uids == NULL )
	{
		fprintf(stderr, "memory leak\n");
		return -1;
	}
	
	fseek(f, 0, SEEK_SET);
	pos = 0;
	while( pos < n )
	{
		if( fgets(sz, MAX_LN_LEN, f) == NULL )
		{
			break;
		}
		
		if( sz[0] != '#' && strlen(sz) == 10 )
		{
			sscanf(sz, "%s", hex);
			hex[4] = '\0';
			hex2int(hex, &id);
			uid = id << 16;
			
			sscanf(&sz[5], "%s", hex);
			hex[4] = '\0';
			hex2int(hex, &id);
			uid += id;
			
			uids[pos++] = uid;
		}
	}
	
	n = pos;
	qsort(uids, n, sizeof(uid), id_cmp);
	
	*p = uids;
	return n;
}

int id_cmp( const void *a, const void *b)
{
	unsigned long xa = *((unsigned long *)a);
	unsigned long xb = *((unsigned long *)b);
	
	if( xa < xb )
	{
		return -1;
	}
	else if( xa > xb )
	{
		return 1;
	}
	else
	{
		return 0;
	}
}

int hex2int(const char *s, int *v)
{
	int i, t;
	
	if( strlen(s) > 8 )
	{
		return -1;
	}
	
	*v = 0;
	for( i=0; i<strlen(s); i++ )
	{
		if( s[i] == '0' )
		{
			t = 0;
		}
		else if( s[i] >= '1' && s[i] <= '9')
		{
			t = (int)(s[i]-'1')+1;
		}
		else if( s[i] >= 'A' && s[i] <= 'F')
		{
			t = (int)(s[i]-'A')+10;
		}
		else if( s[i] >= 'a' && s[i] <= 'f')
		{
			t = (int)(s[i]-'a')+10;
		}
		else
		{
			return -1;
		}
		
		*v = (*v << 4) + t;
	}
	
	return 0;
}

int print_version()
{
	fprintf(stdout, "xyuee %d.%d\n", YUEE_VERSION_MAJOR, YUEE_VERSION_MINOR);
	exit(0);
}

int print_usage()
{
	fprintf(stdout, "Usage: \txyuee [perc], perc is optional, means \"check "
		"mouse state every perc(ms)\"\n"
		"\tValid value in range of %d - %d, default is %d\n\n",
		MIN_SLEEP_TIME/1000, MAX_SLEEP_TIME/1000, DEF_SLEEP_TIME/1000);
	exit(0);
}
